@@ -15,6 +15,9 @@
 # Register the commonly used modules (similar to how pandas does it: https://github.com/pandas-dev/pandas/blob/master/pandas/__init__.py)
 from lux.vis.Clause import Clause
 from lux.core.frame import LuxDataFrame
+from lux.core.sqltable import LuxSQLTable
+from lux.core.joinedsqltable import JoinedSQLTable
+from lux.utils.tracing_utils import LuxTracer
 from ._version import __version__, version_info
 from lux._config import config
 from lux._config.config import warning_format
