@@ -164,13 +164,11 @@ class Compiler:
         from lux.utils.date_utils import is_datetime_string
 
         data_model_lookup = lux.config.executor.compute_data_model_lookup(ldf.data_type)
-
         for vis in vlist:
             for clause in vis._inferred_intent:
                 if clause.description == "?":
                     clause.description = ""
                 # TODO: Note that "and not is_datetime_string(clause.attribute))" is a temporary hack and breaks the `test_row_column_group` example
-                # and not is_datetime_string(clause.attribute):
                 if clause.attribute != "" and clause.attribute != "Record":
                     if clause.data_type == "":
                         clause.data_type = ldf.data_type[clause.attribute]
@@ -273,7 +271,15 @@ class Compiler:
             if measure.aggregation == "":
                 measure.set_aggregation("mean")
             if dim_type == "temporal" or dim_type == "oridinal":
-                return "line", {"x": dimension, "y": measure}
+                if isinstance(dimension.attribute, pd.Timestamp):
+                    # If timestamp, use the _repr_ (e.g., TimeStamp('2020-04-05 00.000')--> '2020-04-05')
+                    attr = str(dimension.attribute._date_repr)
+                else:
+                    attr = dimension.attribute
+                if ldf.cardinality[attr] == 1:
+                    return "bar", {"x": measure, "y": dimension}
+                else:
+                    return "line", {"x": dimension, "y": measure}
             else:  # unordered categorical
                 # if cardinality large than 5 then sort bars
                 if ldf.cardinality[dimension.attribute] > 5:
@@ -366,6 +372,14 @@ class Compiler:
             for attr in relevant_attributes
             if attr != "Record" and attr in ldf._min_max
         )
+        # Replace scatterplot with heatmap
+        if vis.mark == "scatter" and lux.config.heatmap and len(ldf) > lux.config._heatmap_start:
+            vis._postbin = True
+            ldf._message.add_unique(
+                f"Large scatterplots detected: Lux is automatically binning scatterplots to heatmaps.",
+                priority=98,
+            )
+            vis._mark = "heatmap"
         vis._min_max = relevant_min_max
         if auto_channel != {}:
             vis = Compiler.enforce_specified_channel(vis, auto_channel)
