@@ -21,6 +21,7 @@ import warnings
 
 # Suite of test that checks if data_type inferred correctly by Lux
 def test_check_cars():
+    lux.config.set_SQL_connection("")
     df = pd.read_csv("lux/data/car.csv")
     df.maintain_metadata()
     assert df.data_type["Name"] == "nominal"
@@ -38,7 +39,7 @@ def test_check_int_id():
     df = pd.read_csv(
         "https://github.com/lux-org/lux-datasets/blob/master/data/instacart_sample.csv?raw=true"
     )
-    df._repr_html_()
+    df._ipython_display_()
     inverted_data_type = lux.config.executor.invert_data_type(df.data_type)
     assert len(inverted_data_type["id"]) == 3
     assert (
@@ -49,7 +50,7 @@ def test_check_int_id():
 
 def test_check_str_id():
     df = pd.read_csv("https://github.com/lux-org/lux-datasets/blob/master/data/churn.csv?raw=true")
-    df._repr_html_()
+    df._ipython_display_()
     assert (
         "<code>customerID</code> is not visualized since it resembles an ID field.</li>"
         in df._message.to_html()
@@ -62,7 +63,7 @@ def test_check_hpi():
 
     assert df.data_type == {
         "HPIRank": "quantitative",
-        "Country": "nominal",
+        "Country": "geographical",
         "SubRegion": "nominal",
         "AverageLifeExpectancy": "quantitative",
         "AverageWellBeing": "quantitative",
@@ -100,6 +101,22 @@ def test_check_airbnb():
     }
 
 
+def test_check_airports():
+    df = pd.read_csv(
+        "https://raw.githubusercontent.com/altair-viz/vega_datasets/master/vega_datasets/_data/airports.csv"
+    )
+    df.maintain_metadata()
+    assert df.data_type == {
+        "iata": "id",
+        "name": "nominal",
+        "city": "nominal",
+        "state": "geographical",
+        "country": "geographical",
+        "latitude": "quantitative",
+        "longitude": "quantitative",
+    }
+
+
 def test_check_datetime():
     df = pd.DataFrame(
         {
@@ -212,7 +229,7 @@ def test_set_data_type():
         "https://github.com/lux-org/lux-datasets/blob/master/data/real_estate_tutorial.csv?raw=true"
     )
     with pytest.warns(UserWarning) as w:
-        df._repr_html_()
+        df._ipython_display_()
         assert "starter template that you can use" in str(w[-1].message)
         assert "df.set_data_type" in str(w[-1].message)
 
@@ -221,7 +238,7 @@ def test_set_data_type():
     assert df.data_type["Year"] == "nominal"
     with warnings.catch_warnings() as w:
         warnings.simplefilter("always")
-        df._repr_html_()
+        df._ipython_display_()
         assert not w
 
 
@@ -246,7 +263,7 @@ def test_id_with_label():
         "https://github.com/lux-org/lux-datasets/blob/master/data/state_timeseries.csv?raw=true"
     )
     df.maintain_metadata()
-    assert df.data_type == {"Date": "temporal", "State": "nominal", "Value": "quantitative"}
+    assert df.data_type == {"Date": "temporal", "State": "geographical", "Value": "quantitative"}
 
 
 def test_ID_random():
